* ERRHABOR.PRG - generic MDA err-handling proc to force self-destruction
* By M. Asherman, Copyright (c) 2002, SpaceTime Systems
*
* 5/30/02: introduced to avoid growing redundant code in foundation ehm_abort methods.
*
* This program triggers the self-destruction of an object derived from the 
* MDA foundation error-handling classes.  Since the Destroy event may be delayed,
* this is the place to perform cleanup actions that should be done immediately
* and in a deliberate, predictable sequence. 
*
* The return value has no significance, because there is no error checking.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation ehm_abort methods.

lparameters errobj_arg		&& takes 1 required argument
* errobj_arg:		reference to the error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.

local prevonerr

* Set a flag indicating that this object is self-destructing due to user Abort
* or an abnormal error.  This can be used to communicate with the Destroy event logic.
* Note that this flag, like ehp_success, can be propagated by target indirection.
errobj_arg.ehp_aborted = .t.					&& the last change before committing suicide

* perform ON ERROR handling cleanups, if required
* (This is an example of something that needs to happen in a controlled sequence, 
* i.e. before the target is aborted.)
if m.errobj_arg.ehp_on_err_handle ;
  and vartype(m.errobj_arg.ehp_on_err_prev) = 'C'	&& this was the global ON ERROR handler
	prevonerr = m.errobj_arg.ehp_on_err_prev		&& ON ERROR handler before this object was created
	on error &prevonerr							&& restore any previous ON ERROR handler
	m.errobj_arg.ehp_on_err_handle = .F.		&& prevent this from being done again in Destroy or Unload
endif

* support optional "indirection" for copying error info into an external errhandler object
errobj_arg.ehm_copy(m.errobj_arg.ehp_targobj)	&& recurse or just no-op if target is null

* The calling method code is responsible for the final RELEASE THIS or THIS.RELEASE step.
* Note that the assignment errobj_arg.ehp_targobj = NULL may produce an error if done in this proc,
* instead of doing it inline, so we also do that in ehm_abort method code.
