* ERRHCMDP.PRG - ErrHandler core proc to invoke the Diagnostic Command Processor.
* By M. Asherman, Copyright (c) 2002, SpaceTime Systems
*
* 04/23/02: adapted from inline code initially tested in ErrHForm.EHM_CmdProc.
* 04/25/02: moved dialog initialization logic into new ErrHCmdProcDialog.EHCM_Restore method.
* 04/26/02: support the optional WindowState argument.
*
* This function launches or activates a diagnostic command processor associated with 
* the given ErrHandler object, supporting use of THIS and THISFORM in the command line.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation EHM_CmdProc methods.

lparameters errobj_arg, wstate_arg				&& takes 1 required + 1 optional argument
* errobj_arg:				reference to an ErrHandler object.
* wstate_arg:				(optional) windowstate in which to start.
*							If omitted or = .F., let ehp_cmdpwstate determine this.

#include errhandl.h								&& pick up ErrHandler macros

* perform minimal validation to catch basic usage errors (only if SET ASSERT ON, however)
ASSERT vartype(m.errobj_arg) = 'O' MESSAGE program() + " requires vartype(m.errobj_arg) = 'O'"

* check for disabled mode
if not vartype(m.errobj_arg.ehp_cmdproc) $ 'XO'	&& neither NULL nor a valid object ref
	* I could generate an error for this case, but screw it for now.
	return										&& quiet no-op
endif

* check for associated Command Processor dialog already present
if not isnull(m.errobj_arg.ehp_cmdproc)			&& Command Processor dialog already exists
	* support optional explicit WindowState argument
	if vartype(m.wstate_arg) = 'N'				&& explicit WindowState argument supplied
		errobj_arg.ehp_cmdproc.windowstate = m.wstate_arg	&& force into desired zoom state
	endif
	errobj_arg.ehp_cmdproc.show()				&& activate the dialog
	return										&& done
endif

* the Command Processor dialog doesn't yet exist, so launch it now
errobj_arg.ehp_cmdproc = newobject('ErrHCmdProcDialog',ERRH_DIAGD_LIB, '', m.errobj_arg, m.wstate_arg)
errobj_arg.ehp_cmdproc.visible = .t.			&& make the dialog visible
return											&& done
