* ERRHCOPY.PRG - generic MDA err-handling proc to copy detailed error info.
* By M. Asherman, Copyright (c) 1999, SpaceTime Systems
* 11/24/99: introduced for cleaner, more robust approach to indirection.
* 10/26/00: revise to avoid altering the static EHP_AppName property,
*	instead copying the new EHP_AppErrFrom property, for nested usage (see erma0007).
* 11/23/00: added the ehp_aborted property, anticipating its usefulness to Saver, which
*	will want to suppress automatic state-saving upon destruction in these cases.
* 1/22/01: recurse up the target indirection hierarchy, and support optional
*	Abort propagation based on the new ehp_abortstopshere property (erma0017).
*	Also optimize logic to be as fast as possible for the most common, non-error cases.
* 5/29/02: fix bug in Abort propagation logic, which caused failure to abort target unless
*	it had a further indirection target, which should not be required.
*
* This program copies corresponding properties from a source object to a target object,
* where both are derived from MDA foundation error-handling classes.
*
* The return value has no significance, because there is no error checking.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation ehm_copy methods.

lparameters errob1_arg, errob2_arg		&& takes 2 required arguments
* errob1_arg:		reference to the source error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.
* errob2_arg:		reference to the target error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.
*					EHP_... error properties of the target object will be overwritten
*					with corresponding values of source object properties.
*					If this argument is .NULL., just no-op.

* check for no-op case
if isnull(m.errob2_arg)						&& no target object exists
	return									&& quiet no-op
endif

* Optimize logic for the most common non-error cases by copying only if necessary.
* There's no need to copy if we already know both are in an equivalent state of success.
if not m.errob1_arg.EHP_Success or not m.errob2_arg.EHP_Success
	* copy the application-specific error properties
	m.errob2_arg.EHP_AppErrFrom = m.errob1_arg.EHP_AppErrFrom			&& originating application name
	m.errob2_arg.EHP_AppErrCode = m.errob1_arg.EHP_AppErrCode			&& application-specific error #
	m.errob2_arg.EHP_AppErrMsg = m.errob1_arg.EHP_AppErrMsg				&& application-specific error message

	* copy the generic VFP error properties
	m.errob2_arg.EHP_VFPErrNo = m.errob1_arg.EHP_VFPErrNo				&& unique VFP error #
	m.errob2_arg.EHP_VFPErrMethod = m.errob1_arg.EHP_VFPErrMethod		&& method or UDF in which the error occurred
	m.errob2_arg.EHP_VFPErrSourceLineNo = ;
	  m.errob1_arg.EHP_VFPErrSourceLineNo								&& line # within method/UDF that failed
	dimension m.errob2_arg.EHP_VFPErrArray[1]							&& clear extra cruft first
	acopy(m.errob1_arg.EHP_VFPErrArray, m.errob2_arg.EHP_VFPErrArray)	&& array of detailed VFP error info
	m.errob2_arg.EHP_VFPErrMsg = m.errob1_arg.EHP_VFPErrMsg				&& current error msg as a char string
	m.errob2_arg.EHP_VFPErrSource = m.errob1_arg.EHP_VFPErrSource		&& prog line that caused error, if avail
	m.errob2_arg.EHP_VFPErrParam = m.errob1_arg.EHP_VFPErrParam			&& VFP err. msg. param, if applicable
	m.errob2_arg.EHP_VFPErrProgDepth = m.errob1_arg.EHP_VFPErrProgDepth	&& num value of VFP program level of the error 
	m.errob2_arg.EHP_VFPErrProgram = m.errob1_arg.EHP_VFPErrProgram		&& name of the program that produced error
	m.errob2_arg.EHP_VFPErrProgPath = m.errob1_arg.EHP_VFPErrProgPath	&& file path/name of prog that produced error

	* copy the general success/failure and abnormal abort indicators
	m.errob2_arg.EHP_Success = m.errob1_arg.EHP_Success					&& main success flag
	m.errob2_arg.EHP_Aborted = m.errob1_arg.EHP_Aborted					&& user Abort or abnorm err
endif

* see whether to propagate an Abort or directly recurse 
if m.errob1_arg.EHP_Aborted and not m.errob2_arg.ehp_abortstopshere		&& abort is to be propagated
	m.errob2_arg.ehm_abort()											&& also does a recursive ehm_copy
else																	&& not an abort propagation
	if not isnull(m.errob2_arg.ehp_targobj)								&& target has its own target
		errhcopy(m.errob2_arg, m.errob2_arg.ehp_targobj)				&& copy recursively to next target
	endif
endif
