* ERRHDIAG.PRG - ErrHandler core proc to copy diagnostic/tracing/output settings.
* By M. Asherman, Copyright (c) 2000, SpaceTime Systems
* 12/28/00: introduced for convenient propagation of diag/tracing options (see erma0010).
*	Adapted from similar errhcopy.prg.
* 1/9/01: propagate new property ehp_showstack, for stack display (see erma0015).
* 1/28/01: revise specs to omit ehp_interactive from the properties affected (erma0019).
*
* This program copies selected properties from a source object to a destination object,
* where both are derived from MDA foundation error-handling classes.
*
* Only the following properties are copied:
*	ehp_diagforce, ehp_diagmode, ehp_echodebugout, ehp_echologfile, ehp_echoscreen, 
*	ehp_interactive, ehp_log_function, ehp_logfilepath, ehp_pathtypedefault, ehp_showstack, 
*	ehp_tracelevel, and ehp_tracequietly.
*
* The return value has no significance, because there is no error checking.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation ehm_copydiags methods.

lparameters errob1_arg, errob2_arg		&& takes 2 required arguments
* errob1_arg:		reference to the source error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.
*					If this argument is .NULL., just no-op.
* errob2_arg:		reference to the destination error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.
*					Selected EHP_... properties of this object will be overwritten
*					with corresponding values of source object properties.
*					If this argument is .NULL., just no-op.

* perform minimal validation to catch basic usage errors (only if SET ASSERT ON, however)
ASSERT vartype(m.errob1_arg) $ 'OX' MESSAGE program() + " requires vartype(m.errob1_arg) $ 'OX'"
ASSERT vartype(m.errob2_arg) $ 'OX' MESSAGE program() + " requires vartype(m.errob2_arg) $ 'OX'"

* check for no-op cases
if isnull(m.errob1_arg) or isnull(m.errob2_arg)						&& missing object
	return															&& quiet no-op - not an error
endif

* copy the diagnostic mode settings
m.errob2_arg.ehp_diagforce = m.errob1_arg.ehp_diagforce				&& forces use of the diagnostic message dialog
m.errob2_arg.ehp_diagmode = m.errob1_arg.ehp_diagmode				&& enables use of diagnostic message dialog

* copy the output echoing options
m.errob2_arg.ehp_echodebugout = m.errob1_arg.ehp_echodebugout		&& echo messages to the Debugger's output window
m.errob2_arg.ehp_echologfile = m.errob1_arg.ehp_echologfile			&& echo messages to an ASCII text log file
m.errob2_arg.ehp_echoscreen = m.errob1_arg.ehp_echoscreen			&& echo messages to the main VFP window

* copy the error/tracing message logging options
m.errob2_arg.ehp_log_function = m.errob1_arg.ehp_log_function		&& name of app-specific logging function
m.errob2_arg.ehp_logfilepath = m.errob1_arg.ehp_logfilepath			&& path of ASCII text log file
m.errob2_arg.ehp_pathtypedefault = m.errob1_arg.ehp_pathtypedefault	&& code for default flavor of absolute/relative path 

* copy the tracing and verbosity options
m.errob2_arg.ehp_showstack = m.errob1_arg.ehp_showstack				&& verbosity of errhvfms, as to program stack
m.errob2_arg.ehp_tracelevel = m.errob1_arg.ehp_tracelevel			&& level of diagnostic tracing in effect
m.errob2_arg.ehp_tracequietly = m.errob1_arg.ehp_tracequietly		&& suppress tracing messages, except optional echoing 
