* ERRHSHOW.PRG - proc for reporting app errors in MDA generic error-handling classes
* By M. Asherman, Copyright (c) 1999 - 2000, SpaceTime Systems
* 11/23/99: adapted from logic in ErrHandler.EHM_ShowAppErr.
* 11/24/99: Revised specs pertaining to indirection, for greater symmetry and robustness.
*	Also set the ehp_success flag here.
* 10/26/00: Capture the current this.EHP_AppName in new property ehp_apperrfrom
*	for the benefit of target indirection in nested ErrHandler cases.  
*	The target object is responsible for reporting ehp_apperrfrom, if desired, 
*	but this is entirely optional.  
*	Also suppress display of message if ehp_quiet is set (see erma0007).
* 11/17/00: revise specs for empty ernum_arg cases to support warnings and diagnostic msgs,
*	instead of the previous treatment of this as a no-op (see erma0004).
*	Also beef up arg-checking, in case of basic usage errors.
* 11/17-19/00: add logic for tie-in to the diagnostic message dialog, controlled by
*	new properties, ehp_diagmode and ehp_diagforce.  Initialize ehp_diagresponse = ''.
*	Support 3 additional optional arguments for window title and whether to allow
*	Ignore and Retry commands in the diagnostic dialog.
* 11/20/00: include app name in default tracing message title/prefix.
* 11/25/00: support new message echoing options: ehp_echodebugout, ehp_echoscreen,
*	ehp_tracequietly.
* 11/28/00: revise to handle Abort logic here, rather than in diagnostic dialog.
*	This requires introducing a new optional argument, aborto_arg.
*	Also add more argument validation logic, to keep this as robust as possible.
* 11/29/00: introduce the new mode_arg for forcing diag dialog or WAIT WINDOW NOWAIT.
*	Revise specs to always treat an Abort as a type of failure, supplying a general
*	user abort code (ERRH_EC_SHOW_ABORT) & msg if there was no previous error.
* 11/30-12/1/00: added logic for echoing to ASCII text log file.
*	Also fix logic for cases when message is empty and for ehp_quiet cases,
*	which should not affect echoing to secondary outputs.
* 12/2/00: add provisions for optional application-dependent logging function.
*	Also eliminate special-case treatment of empty messages as no-ops,
*	because this is a needless complication of logic, of questionable value.
*	Eliminate property ehp_messageboxtype by using new ERRH_MESSBOX_ICON_ERR/INF macros instead.
* 12/3/00: substitute calls to ehm_copy() for errhcopy().  Avoid direct calls to core UDFs.
*	Translate chr(13) to chr(13) + chr(10) for file output cases, for better text display.
* 1/3/01: watch out for self-destruction while the modal diagnostic dialog is active,
*	e.g. triggered by closing the main VFP window -> ON SHUTDOWN handler.
* 1/12/01: revise specs for aborto_arg to return to caller of EHM_ShowAppErr
*	by default, not one level deeper as done before (see erma0016).
* 2/12/02: fix logic to return .F. in non-local RETURN TO ... failure cases (erma0024).
*	Also adjust logic to include window caption info at start of editbox text (erma0022).
* 3/27/02: added fix to avoid errors caused by attemps to RETURN TO ON... commands.
* 10/1/02: fix logic to avoid non-local RETURN TO ... on Aborts when abort_to = 0.
*
* The function return value of this procedure has no significance, but
* this procedure effectively returns a value in the ehp_diagresponse property of
* the ErrHandler object specified by the first argument.
* (See ERRH_DIAGRESP_... macros of user responses codes for the diagnostic dialog.)
* The empty string means that the user replied OK, or there was no user interaction.
* The caller is reponsible for taking appropriate action (e.g. Retry or Ignore).
*
* This program operates in both interactive and non-interactive modes, where
* non-interactive mode always avoids pausing for a user interaction.
* Messages in non-interactive mode are always generated via WAIT WINDOW NOWAIT,
* so they may be truncated to avoid exceeeding the 254 character limit.
*
* In interactive modes, a WAIT WINDOW is used for short messages (<= 254 chars),
* and MESSAGEBOX() is used for long messages.  There is also a diagnostic dialog mode.
*
* Custom properties are used to retain the application error # and message.
* Additional VFP error information may be available in "EHP_VFPErr..." properties,
* but this depends on the application-specific error case conventions. 
*
* This procedure performs argument validation, but it only gives warnings for
* any errors it detects.  From the caller's standpoint, this procedure always succeeds.
* If the user Aborts, the caller will normally not see the return from ehp_showapperr, 
* because this procedure will directly RETURN TO a lower level of the stack.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
*
* This core proc should only be called via EHM_ShowAppErr methods, not directly,
* in order to allow for selective redefinition of the method by subclassing.
* Also note that the default Warning/Tracing message title or prefix assumes this,
* and Abort logic also depends on relative stack depth assumptions.

* support 1 - 8 arguments
lparameters errobj_arg, ernum_arg, msg_arg, title_arg, mode_arg, ignore_arg, retry_arg, aborto_arg
* errobj_arg:		reference to the error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.
* ernum_arg:		application-specific numeric error code, if non-empty.
*					This code is saved in the ehp_apperrcode property, if non-zero.
*					If this value is 0 or empty, just display a warning, tracing, or
*					informative message, but don't treat it as an error.
* msg_arg:			text of the message to be displayed (and saved to ehp_apperrmsg
*					if ernum_arg is non-zero).
* title_arg:		optional dialog title/WAIT WINDOW message prefix.
*					If empty or omitted, a generic default is automatically supplied.
*					For application errors (ernum_arg non-zero), the default title 
*					includes the application name and the error number.
*					For tracing messages (ernum_arg = 0), the default title includes
*					the application name and the calling program name.
* mode_arg:			optional number code indicating whether to force the use
*					of a different mode of output from the default.
*					If empty or 0, let the mode be determined automatically,
*					based on property settings.  Otherwise, the following options
*					are supported (macro-defined codes are in errhandl.h):
*					ERRH_SHOWMODE_DIAGFORCE: forces use of the diagnostic dialog,
*					ERRH_SHOWMODE_NOWAIT: forces use of WAIT WINDOW NOWAIT.
* ignore_arg:		optional flag indicating whether to allow the user to choose 
*					the Ignore option in the diagnostic message dialog.
*					This has no bearing on whether the diagnostic dialog is used,
*					but only on whether its Ignore button will be enabled.
* retry_arg:		optional flag indicating whether to allow the user to choose 
*					the Retry option in the diagnostic message dialog.
*					This has no bearing on whether the diagnostic dialog is used,
*					but only on whether its Retry button will be enabled.
* aborto_arg:		optional abort-to-depth argument, indicating the relative
*					program level to which an Abort should return if the 
*					diagnostic dialog is used and the user selects Abort.
*					If omitted or 0, abort will RETURN TO the method or UDF
*					that invoked ehm_showapperr.  Otherwise this number
*					specifies a different stack depth relative to the default.
*					E.g. -1 returns to the program that called the program that
*					called ehm_showapperr, etc.
*					(This argument is primarily for internal use by the core
*					error handling procedures, errherr and errhoerr.prg.)

#include errhandl.h							&& pick up macros for error codes and messages

local msg, errnum, prognm, msgtitle, mode, ignorable, retriable, abort_to, ;
  msgprefix, logmsg, logmsglen, nwritten, ;
  usedialog, dlg, logexp, prevsucc, prevtrap, success, ;
  waitmsg, msglen, nowaitopt, ;
  retdepth, returnto, periodoff, retval

* perform some minimal argument validation in case of basic usage errors
if vartype(m.errobj_arg) # 'O' ;
  or (vartype(m.errobj_arg) = 'O' ;
  and vartype(m.errobj_arg.ehp_apperrcode) # 'N')	&& not a valid ErrHandler object reference
	* use crude error handling, since we don't have reference to an error-handling object
	wait window "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVARG1)) + ': ' ;
	  + ERRH_EM_SHOW_INVARG1
	return									&& treat as a non-fatal no-op - continue after warning
endif
m.errobj_arg.ehp_diagresponse = ''			&& initialize result of diagnostic dialog

* validate the message string argument
if vartype(m.msg_arg) # 'C' ;
  and (vartype(m.msg_arg) = 'O' or not empty(m.msg_arg))	&& invalid msg_arg
	* construct a warning message, since we are not treating this as a separate error
	msg = "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVMSG)) + ': ' ;
	  + ERRH_EM_SHOW_INVMSG
else										&& the caller supplied a valid message
	msg = alltrim(iif(empty(m.msg_arg), '', ;
	  m.msg_arg))							&& empty string or caller-supplied message
endif

* validate the error code argument
if vartype(m.ernum_arg) # 'N' ;
  and (vartype(m.ernum_arg) = 'O' or not empty(m.ernum_arg))	&& invalid error # arg
	* construct a warning message, since we are not treating as an error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVERNO)) + ': ' ;
	  + ERRH_EM_SHOW_INVERNO
	errnum = 0								&& avoid bombing on invalid arg
else										&& the caller supplied a valid message
	errnum = iif(empty(m.ernum_arg), 0, m.ernum_arg)	&& 0 or caller-supplied #
endif

* validate the optional title argument
if vartype(m.title_arg) # 'C' ;
  and (vartype(m.title_arg) = 'O' or not empty(m.title_arg))	&& invalid title_arg
	* construct a warning message, since we are not treating this as a separate error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVTITLE)) + ': ' ;
	  + ERRH_EM_SHOW_INVTITLE
	msgtitle = ''							&& let default title be used
else										&& the caller supplied a valid title
	msgtitle = alltrim(iif(empty(m.title_arg), '', ;
	  m.title_arg))							&& empty string or caller-supplied title
endif

* validate the output mode argument
if vartype(m.mode_arg) # 'N' ;
  and (vartype(m.mode_arg) = 'O' or not empty(m.mode_arg))	&& invalid mode arg
	* construct a warning message, since we are not treating as an error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVMODE)) + ': ' ;
	  + ERRH_EM_SHOW_INVMODE
	mode = 0								&& avoid bombing on invalid number
else										&& the caller supplied a valid message
	mode = iif(empty(m.mode_arg), 0, m.mode_arg)	&& 0 or caller-supplied #
	if not inlist(m.mode, ;
	  ERRH_SHOWMODE_DEFAULT, ;
	  ERRH_SHOWMODE_DIAGFORCE, ;
	  ERRH_SHOWMODE_NOWAIT)
		* construct a warning message, since we are not treating as an error
		msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
		  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVMODE)) + ': ' ;
		  + ERRH_EM_SHOW_INVMODE
		mode = 0							&& avoid bombing on invalid number
	endif
endif

* validate the allow-ignore argument
if vartype(m.ignore_arg) # 'L'				&& invalid allow-ignore flag
	* construct a warning message, since we are not treating as an error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVIGNO)) + ': ' ;
	  + ERRH_EM_SHOW_INVIGNO
	ignorable = .f.							&& avoid bombing
else										&& the caller supplied a valid message
	ignorable = m.ignore_arg				&& caller-supplied flag
endif

* validate the allow-retry argument
if vartype(m.retry_arg) # 'L'				&& invalid allow-retry flag
	* construct a warning message, since we are not treating as an error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVRETRY)) + ': ' ;
	  + ERRH_EM_SHOW_INVRETRY
	retriable = .f.							&& avoid bombing
else										&& the caller supplied a valid message
	retriable = m.retry_arg					&& caller-supplied flag
endif

* validate the abort-to-depth argument
if vartype(m.aborto_arg) # 'N' ;
  and (vartype(m.aborto_arg) = 'O' or not empty(m.aborto_arg))	&& invalid abort-to-depth
	* construct a warning message, since we are not treating as an error
	msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
	  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_INVABORT)) + ': ' ;
	  + ERRH_EM_SHOW_INVABORT
	abort_to = 0							&& avoid bombing on non-numeric value
else										&& the caller supplied a valid message
	abort_to = iif(empty(m.aborto_arg), 0, m.aborto_arg)	&& 0 or caller-supplied #
endif

* see whether we have an application error or just a warning/tracing/info message
if empty(m.errnum)							&& warning or tracing message
	* Prepare default prefix or title of message to be displayed.
	* Assume that the caller of interest is two levels back in the stack,
	* i.e. the guy that called the EHM_ShowAppErr method.
	prognm = iif(empty(m.errobj_arg.ehp_appname), '', m.errobj_arg.ehp_appname + ' ') ;
	  + program(max(0, program(-1) - 2))	&& calling program name for prefix or title

	* supply a default Trace dialog title or caller-supplied title
	msgtitle = iif(empty(m.msgtitle), m.prognm + " Tracing", m.msgtitle)
else										&& application error
	* Save application-defined error code and message properties.
	* Whether there is additional VFP error info depends on app conventions.
	m.errobj_arg.ehp_apperrfrom = m.errobj_arg.EHP_AppName	&& save originating app name
	m.errobj_arg.ehp_apperrcode = m.errnum	&& app-specific err #
	m.errobj_arg.ehp_apperrmsg = m.msg		&& app-specific err message text
	m.errobj_arg.ehp_success = .f.			&& treat this as an error

	* support optional "indirection" for copying error info into an external errhandler object
	m.errobj_arg.ehm_copy(m.errobj_arg.ehp_targobj)	&& just no-op if target is null

	* prepare prefix or title of message to be displayed
	prognm = iif(empty(m.errobj_arg.ehp_appname), ;
	  proper(m.errobj_arg.EHP_VFPErrMethod), ;
	  m.errobj_arg.ehp_appname)				&& application name for prefix or title

	* supply a default Trace dialog title or caller-supplied title
	msgtitle = iif(empty(m.msgtitle), ;
	  m.prognm + " Error # " + ltrim(str(m.errnum)), m.msgtitle)
endif

* support message echoing options
if m.errobj_arg.ehp_echologfile				&& echoing to log file is enabled
	* Generate output via VFP's handy new builtin string-to-file function,
	* which conveniently avoids triggering errors, instead simply returning the
	* number of bytes written, from which errors can be deduced.
	logmsg = '@ ' + dtoc(date()) + ' ' + time() ;
	  + ': ' + m.msgtitle + chr(13) + chr(10) + strtran(m.msg, chr(13), chr(13) + chr(10)) ;
	  + chr(13) + chr(10) + chr(13) + chr(10)
	logmsglen = len(m.logmsg)				&& length of message to be written to log file
	nwritten = STRTOFILE(m.logmsg, m.errobj_arg.ehp_logfilepath, ;
	  .t.) 									&& append string directly to text file
	* check for errors in writing to log file, e.g. due to invalid path or access conflict
	if m.nwritten < m.logmsglen				&& too few bytes were written
		* tack on a warning message about inability to echo to log file
		msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
		  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_LOGWRITE)) + ': ' ;
		  + ERRH_EM_SHOW_LOGWRITE + m.errobj_arg.ehp_logfilepath
	endif
endif
if m.errobj_arg.ehp_echodebugout			&& echoing via DEBUGOUT is enabled
	* generate output to the VFP Debugger's Output window, if open (else no-op)
	DEBUGOUT m.msgtitle + ": " + m.msg
endif
if m.errobj_arg.ehp_echoscreen				&& echoing to SCREEN is enabled
	* generate output to the main VFP window, with conversion for proper newlines
	ACTIVATE SCREEN							&& direct ? output to _screen
	? m.msgtitle + ": " + strtran(m.msg, chr(13), chr(10))
endif

* determine whether diagnostic dialog will be used
do case
  case m.mode = ERRH_SHOWMODE_NOWAIT
  	usedialog = .f.							&& never use dialog in NOWAIT mode
  case m.mode = ERRH_SHOWMODE_DIAGFORCE
  	usedialog = .t.							&& always use dialog in DIAGFORCE mode
  otherwise									&& let property settings determine mode
  	* only use diagnostic dialog if it's enabled and not suppressed
  	usedialog = m.errobj_arg.ehp_diagforce ;
  	  or (m.errobj_arg.ehp_diagmode ;
  	  and m.errobj_arg.ehp_interactive ;
  	  and not m.errobj_arg.ehp_quiet ;
  	  and not (empty(m.errnum) and m.errobj_arg.ehp_tracequietly))
endcase

* invoke diagnostic message dialog if enabled for this call
if m.usedialog								&& use dialog instead of WAIT or MESSAGEBOX
	* use diag dialog to show the message (repeat title in message for easy copying)
	dlg = newobject(ERRH_DIAGD_CLASS, ERRH_DIAGD_LIB, '', m.errobj_arg, ;
	  m.msgtitle + chr(13) + m.msg, ;
	  m.msgtitle, m.ignorable, m.retriable)
	dlg.show								&& make the modal dialog visible
	SET MESSAGE TO							&& clear any lingering status bar cruft

	* check for self-destruction cases, e.g. because user clicked VFP's close box
	if vartype(m.errobj_arg) # 'O'			&& she's gone
		* nothing more can be done without the ErrHandler object reference
		return								&& allow app to terminate ASAP
	endif

	* Support message echoing options for user replies other than OK
	if not empty(m.errobj_arg.ehp_diagresponse)	&& response other than OK
		if m.errobj_arg.ehp_echologfile				&& echoing to log file is enabled
			* Generate output via VFP's handy new builtin string-to-file function.
			logmsg = '@ ' + dtoc(date()) + ' ' + time() ;
			  + ': User Reply: ' + m.errobj_arg.ehp_diagresponse ;
			  + chr(13) + chr(10) + chr(13) + chr(10)
			logmsglen = len(m.logmsg)				&& length of message to be written to log file
			nwritten = STRTOFILE(m.logmsg, m.errobj_arg.ehp_logfilepath, ;
			  .t.) 									&& append string directly to text file
			* check for errors in writing to log file, e.g. due to invalid path or access conflict
			if m.nwritten < m.logmsglen				&& too few bytes were written
				* just give a crude warning message about inability to echo to log file
				waitmsg = "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_LOGWRITE)) + ': ' ;
				  + ERRH_EM_SHOW_LOGWRITE + m.errobj_arg.ehp_logfilepath
				if len(m.waitmsg) > ERRH_MAXWAITWINDLEN			&& exceeds limit for WAIT WINDOW
					waitmsg = substr(m.waitmsg, 1, ERRH_MAXWAITWINDLEN - 3) + "..."
				endif
				WAIT WINDOW m.waitmsg				&& display warning
			endif
		endif
		if m.errobj_arg.ehp_echodebugout	&& echoing via DEBUGOUT is enabled
			* generate output to the VFP Debugger's Output window, if open (else no-op)
			DEBUGOUT "User Reply: " + m.errobj_arg.ehp_diagresponse
		endif
		if m.errobj_arg.ehp_echoscreen		&& echoing to SCREEN is enabled
			* generate output to the main VFP window
			ACTIVATE SCREEN					&& direct ? output to _screen
			? "User Reply: " + m.errobj_arg.ehp_diagresponse
		endif
	endif

	* support optional post-dialog application-dependent logging via external API
	if vartype(m.errobj_arg.ehp_log_function) = 'C' ;
	  and not empty(m.errobj_arg.ehp_log_function)	&& function name specified - feature is enabled
		* construct the logging expression to be evaluated, assuming conformance to standard API
		logexp = m.errobj_arg.ehp_log_function + '(m.errobj_arg, m.errnum, m.msg, m.msgtitle)'

		* Use the ehp_traperrors mechanism to cover API logging errors that occur
		* while reporting application-level errors and tracing messages, which we
		* can handle in most normal cases.
		prevsucc = m.errobj_arg.ehp_success			&& hold on to previous success flag
		m.errobj_arg.ehp_success = .t.				&& temporarily clear the success flag 
		prevtrap = m.errobj_arg.ehp_traperrors		&& hold on to previous setting
		m.errobj_arg.ehp_traperrors = .t.			&& enable error trapping via errherr

		* attempt to perform API-based logging, anticipating possible errors at this step 
		success = evaluate(m.logexp)				&& invoke application-depending logging function
		m.errobj_arg.ehp_traperrors = m.prevtrap	&& restore previous setting of trapping

		* check for API logging errors
		if not m.success or not m.errobj_arg.ehp_success	&& API logging failed
			* generate a warning message about inability to do application-depending logging
			waitmsg = "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_LOGAPI)) + ': ' ;
			  + ERRH_EM_SHOW_LOGAPI + chr(13) + m.logexp
			if len(m.waitmsg) > ERRH_MAXWAITWINDLEN			&& exceeds limit for WAIT WINDOW
				waitmsg = substr(m.waitmsg, 1, ERRH_MAXWAITWINDLEN - 3) + "..."
			endif
			WAIT WINDOW m.waitmsg							&& display warning
		endif

		* restore previous success flag, because API logging error was treated as a warning
		m.errobj_arg.ehp_success = m.prevsucc				&& make no assumption about context
		m.errobj_arg.ehm_copy(m.errobj_arg.ehp_targobj)		&& support target indirection
	endif

	* check for an explicit Abort request
	if m.errobj_arg.ehp_diagresponse == ERRH_DIAGRESP_ABORT		&& asked for it
		* see if we already have an error
		if m.errobj_arg.ehp_success								&& no error yet
			m.errobj_arg.ehp_apperrfrom = m.errobj_arg.EHP_AppName	&& save originating app name
			m.errobj_arg.ehp_apperrcode = ERRH_EC_SHOW_ABORT	&& generic user Abort code
			m.errobj_arg.ehp_apperrmsg = ERRH_EM_SHOW_ABORT		&& generic user Abort message
			m.errobj_arg.ehp_success = .f.						&& treat this as an error
		endif
		
		* take immediate action to Abort here, so caller doesn't need to handle this
		m.errobj_arg.ehm_abort				&& invoke the ErrHandler's suicide method
		if m.abort_to = 0					&& no extra stack-popping was requested
			return							&& avoid risky use of non-local RETURN TO ...
		endif
		
		* Prepare to pop the stack to the level (or below) where an error was trapped
		* or a tracing/warning/information message was issued.
		retdepth = max(0, program(-1) ;
		  + m.abort_to - 2)					&& program depth to return to

		* skip over any ON... command levels (MDA fix of 3/27/02)
		do while m.retdepth > 1 and program(m.retdepth) == 'ON... '	&& prog() returns this for all ON commands
			* this is correct, because ON commands always execute a single command line
			retdepth = m.retdepth - 1		&& pop back an extra level below this ON...
		enddo

		if m.retdepth < 2					&& called from VFP command window or top-level program
			RETURN TO MASTER .F.			&& return to the highest program level available
		endif

		* Get program name, checking for imbedded period in method names, which causes a syntax error.
		* (See notes in errherr.prg about flaws in VFP's RETURN TO <program> command.)
		returnto = program(m.retdepth)		&& name of program that called the one encountering error
		periodoff = rat('.', m.returnto)	&& offset of last period
		if m.periodoff > 0					&& period found
			returnto = substr(m.returnto, m.periodoff + 1)	&& keep only the last portion
		endif

		* make a non-local RETURN .F. to the appropriate stack depth from which to continue
		retval = .F.						&& use a variable instead of literal to avoid syntax error (!)
		RETURN TO &returnto m.retval		&& bypasses code if program was taken by surprise
	endif

	* not an Abort request - let the caller be responsible for checking ehp_diagresponse
	return									&& done - diag dialog case
endif

* support optional application-depending logging for non-dialog cases
if vartype(m.errobj_arg.ehp_log_function) = 'C' ;
  and not empty(m.errobj_arg.ehp_log_function)	&& function name specified - feature is enabled
	* construct the logging expression to be evaluated, assuming conformance to standard API
	logexp = m.errobj_arg.ehp_log_function + '(m.errobj_arg, m.errnum, m.msg, m.msgtitle)'

	* Try to handle errors due to invalid ehp_log_function caused by 
	* unhandled errors within the application-dependent API function logic.
	* There's no use in an ON ERROR handler in this context, because VFP
	* doesn't support nested error handlers, and it's the Error event that
	* will be triggered if no error handling is currently in progress.
	* This means that we can't cleanly handle API logging errors in all cases,
	* so API function must take responsibility for being robust, and the usage
	* of ehp_log_function must be carefully tested.

	* Use the ehp_traperrors mechanism to cover API logging errors that occur
	* while reporting application-level errors and tracing messages, which we
	* can handle in most normal cases.
	prevsucc = m.errobj_arg.ehp_success			&& hold on to previous success flag
	m.errobj_arg.ehp_success = .t.				&& temporarily clear the success flag 
	prevtrap = m.errobj_arg.ehp_traperrors		&& hold on to previous setting
	m.errobj_arg.ehp_traperrors = .t.			&& enable error trapping via errherr

	* attempt to perform API-based logging, anticipating possible errors at this step 
	success = evaluate(m.logexp)				&& invoke application-depending logging function
	m.errobj_arg.ehp_traperrors = m.prevtrap	&& restore previous setting of trapping

	* check for API logging errors
	if not m.success or not m.errobj_arg.ehp_success	&& API logging failed
		* tack on a warning message about inability to do application-depending logging
		msg = m.msg + iif(empty(m.msg), '', chr(13)) ;
		  + "ERRHSHOW Warning # " + ltrim(str(ERRH_EC_SHOW_LOGAPI)) + ': ' ;
		  + ERRH_EM_SHOW_LOGAPI + chr(13) + m.logexp
	endif

	* restore previous success flag, because API logging error was treated as a warning
	m.errobj_arg.ehp_success = m.prevsucc				&& make no assumption about context
	m.errobj_arg.ehm_copy(m.errobj_arg.ehp_targobj)		&& support target indirection
endif

* check for quiet mode cases
if m.errobj_arg.EHP_quiet ;
  or (empty(m.errnum) ;
  and m.errobj_arg.ehp_tracequietly)		&& no further output is required
	SET MESSAGE TO							&& clear any lingering status bar cruft
	return									&& done
endif

* not using diagnostic dialog - check for long message cases
msgprefix = m.msgtitle + ": "				&& prefix for WAIT WINDOW format, if used
waitmsg = m.msgprefix + m.msg				&& assemble tentative WAIT WINDOW message
msglen = len(m.waitmsg)						&& length of message as supplied
if m.msglen > ERRH_MAXWAITWINDLEN			&& exceeds limit for WAIT WINDOW
	if m.errobj_arg.ehp_interactive and m.mode # ERRH_SHOWMODE_NOWAIT	&& interactive mode
		* use messagebox() instead of WAIT WINDOW for long messages
		wait clear							&& in case of any WAIT WINDOW residue
		
		* generate an informative title for the modal messagebox dialog
		messagebox(m.msg, ;
		  iif(empty(m.errnum), ERRH_MESSBOX_ICON_INF, ERRH_MESSBOX_ICON_ERR), ;
		  m.msgtitle)						&& display modal message dialog
	else									&& non-interactive case
		* truncate the message to fit WAIT WINDOW NOWAIT without generating an error
		msg = substr(m.waitmsg, 1, ERRH_MAXWAITWINDLEN - 3) + "..."
		WAIT WINDOW NOWAIT m.msg			&& display message without pausing
	endif
else										&& not a long message
	* use the minimally obtrusive WAIT WINDOW, with no pause if non-interactive
	nowaitopt = iif(m.errobj_arg.ehp_interactive and m.mode # ERRH_SHOWMODE_NOWAIT, ;
	  '', 'NOWAIT')
	WAIT WINDOW &nowaitopt m.waitmsg		&& display message in a wait window
endif

* Clear any lingering status bar prompt, as a general convenience.
* We can always make this step optional if it becomes necessary.
SET MESSAGE TO								&& in case status bar had in-progress msg
