* ERRHVFMS.PRG - MDA err-handling UDF to return text of fully expanded VFP error message
* By M. Asherman, Copyright (c) 1999, SpaceTime Systems
* 11/6/99: initial implementation (in ErrHandler.EHM_VFPErrMsg) just does the easy stuff.
* 11/7/99: renamed this generic VFP-related method to start with "VFP", for clearer
*	distinction from application-specific methods.  Also display additional info
*	obtained from program() and sys(16).
* 11/23/99: broke out old logic from ErrHandler.EHM_VFPErrMsg.
* 1/9/01: support new property ehp_showstack, for stack display (see erma0015).
*	Adapted logic from old MDA110's shocalls.prg.
* 1/15/01: tweak program stack layout for better readability.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation EHM_VFPErrMsg methods.
*
* As a further refinement, we'd want to display additional details that can be 
* gleaned from AERROR() results in property EHP_VFPErrArray, but this can wait.
*
* Note that extended stack information, unlike the rest of info displayed,
* is obtained dynamically at the time this function is called.
* It may be desirable to capture the stack info in an array property of ErrHandler,
* instead of using the present logic, in order to assure that this really reflects
* the stack at the point where the error occurred, not the point where it's reported.
* Fortunately, however, the present approach seems to work fine in typical usage.

function errhvfms
* returns a character string result, the decoded VFP error message details.

lparameters errobj_arg		&& takes 1 required argument
* errobj_arg:		reference to the error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.

local msg, n

* assemble a basic generic error message
msg = "VFP Error #: " + ltrim(str(m.errobj_arg.EHP_VFPErrNo)) ;
  + ", " + alltrim(m.errobj_arg.EHP_VFPErrMsg) ;
  + chr(13) + "Line #: " + ltrim(str(m.errobj_arg.EHP_VFPErrSourceLineNo)) ;
  + ", In: " + alltrim(m.errobj_arg.EHP_VFPErrMethod) ;
  + chr(13) + "Source: " + alltrim(m.errobj_arg.EHP_VFPErrSource) ;
  + chr(13) + "ProgPath: " + alltrim(m.errobj_arg.EHP_VFPErrProgPath) ;
  + chr(13) + 'Program(' + ltrim(str(m.errobj_arg.EHP_VFPErrProgDepth)) + '): ' ;
  + m.errobj_arg.EHP_VFPErrProgram

* support optional non-verbose display, omitting the program stack
if not m.errobj_arg.ehp_showstack
	return m.msg	&& done - return message string without additional stack info
endif

* tack on the full program stack
for n = m.errobj_arg.EHP_VFPErrProgDepth - 1 to 1 step -1
	msg = m.msg + chr(13) + 'Program(' + ltrim(str(m.n)) + '): ' + program(m.n)
endfor

return m.msg	&& return message string - caller decides if/how to display it
