* ERRHZERO.PRG - generic MDA err-handling proc to clear (zero out) error info.
* By M. Asherman, Copyright (c) 1999, SpaceTime Systems
* 11/24/99: introduced for cleaner, more robust approach to indirection.
* 10/26/00: clear the new ehp_apperrfrom property (see erma0007).
* 12/3/00: substitute calls to ehm_copy() for errhcopy().  Avoid direct calls to core UDFs.
* 1/12/01: revise specs to include EHP_Aborted among properties cleared (erma0016).
*
* This program resets all of the error detail properties for a given object,
* which should be derived from the MDA foundation error-handling classes.
*
* If indirection is enabled (ehp_targobj non-null), the target errhandler's
* error properties are also cleared.
*
* We use a PRG for this logic so it can be shared across multiple base classes,
* which can't have a common ancestor class in VFP.
* This core program should only be called directly by foundation ehm_zero methods.

lparameters errobj_arg		&& takes 1 required argument
* errobj_arg:		reference to the error-handling object, which should be 
*					derived from one of the MDA foundation error-handling classes.

* reset the general success/failure indicator and abort flag
m.errobj_arg.EHP_Success = .t.					&& main success flag - no error state
m.errobj_arg.EHP_Aborted = .f.					&& clear the abnormal Abort indicator

* wipe the application-specific error properties (but not the default application name)
m.errobj_arg.EHP_AppErrFrom = ''				&& originating application name
m.errobj_arg.EHP_AppErrCode = 0					&& application-specific error #
m.errobj_arg.EHP_AppErrMsg = ''					&& application-specific error message

* wipe the generic VFP error properties
m.errobj_arg.EHP_VFPErrNo = 0					&& unique VFP error #
m.errobj_arg.EHP_VFPErrMethod = ''				&& method or UDF in which the error occurred
m.errobj_arg.EHP_VFPErrSourceLineNo = 0			&& line # within method/UDF that failed
dimension m.errobj_arg.EHP_VFPErrArray[1]		&& array of detailed VFP error info
m.errobj_arg.EHP_VFPErrArray[1] = 0				&& VFP error # is alwaysthe 1st element
m.errobj_arg.EHP_VFPErrMsg = ''					&& current error msg as a char string
m.errobj_arg.EHP_VFPErrSource = ''				&& prog line that caused error, if avail
m.errobj_arg.EHP_VFPErrParam = ''				&& VFP err. msg. param, if applicable
m.errobj_arg.EHP_VFPErrProgDepth = 0			&& num value of VFP program level of the error 
m.errobj_arg.EHP_VFPErrProgram = ''				&& name of the program that produced error
m.errobj_arg.EHP_VFPErrProgPath = ''			&& file path/name of prog that produced error

* support optional "indirection" for copying error info into an external errhandler object
m.errobj_arg.ehm_copy(m.errobj_arg.ehp_targobj)	&& just no-op if target is null
